@@ -321,16 +321,17 @@ class Cluster(object):
         """.format(name, type(self).__name__,  similar_objects)
 
                 raise AttributeError(ERROR_MSG)
+
+            # Setting the account or partition name will automatically result in
+            # activating the corresponding flags
+            if name.endswith("_name"):
+                key = "use_{}".format(name.split("_")[0])
+                self.__dict__[key] = True
         else:
             super(Cluster, self).__setattr__(name, value)
 
 
-        # Setting the account or partition name will automatically result in
-        # activating the corresponding flags
-        if name.endswith("_name"):
-            key = "use_{}".format(name.split("_")[0])
-            self.__dict__[key] = True
-        
+                
         
         
     def ExecuteCMD(self, cmd, raise_error = False, return_output = False, on_cluster = False):
@@ -621,12 +622,16 @@ class Cluster(object):
 
                 list_of_inputs.append(input_file)     
                 list_of_outputs.append(output_file)
-            except:
+            except Exception as e: 
                 MSG = '''
 Error while writing input file {}.
+
+Error message:
 '''.format(label)
+                MSG += str(e)
                 print(MSG)
 
+
             # Release the lock on the threads
             self.lock.release()            
             
@@ -1511,10 +1516,11 @@ Error while connecting to the cluster to copy the files:
                     if error_struct > 1e-2:
                         print("ERROR IDENTIFYING STRUCTURE!")
                         MSG = """
-Error in thread {}.
-     Displacement between the expected structure {} and the one readed from the calculator
-     is of {} A.
-""".format(threading.get_native_id(), jobs_id[i], error_struct)
+                            Error in thread {}.
+                            Displacement between the expected structure {} 
+                            and the one readed from the calculator
+                            is of {} A.
+                        """.format(threading.get_native_id(), jobs_id[i], error_struct)
                         print(MSG)
                         ensemble.structures[jobs_id[i]].save_scf('t_{}_error_struct_generated_{}.scf'.format(threading.get_native_id(), jobs_id[i]))
                         structures[i].save_scf('t_{}_error_struct_cmp_local_{}.scf'.format(threading.get_native_id(), jobs_id[i]))
