@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from __future__ import print_function
+from __future__ import print_function, annotations
 
 """
 This module performs the relax over more
@@ -15,6 +15,7 @@ import sscha.Cluster
 import sscha.Utilities as Utilities
 import cellconstructor as CC
 import cellconstructor.symmetries
+from sscha.aiida_ensemble import AiiDAEnsemble
 
 import sys, os
 
@@ -49,8 +50,17 @@ __ALLOWED_KEYS__ = [__RELAX_TYPE__, __RELAX_NCONFIGS__, __RELAX_MAX_POP__,
 
 class SSCHA(object):
 
-    def __init__(self, minimizer = None, ase_calculator=None, N_configs=1, max_pop = 20,
-                 save_ensemble = False, cluster = None, **kwargs):
+    def __init__(
+        self,
+        minimizer = None,
+        ase_calculator=None, 
+        aiida_inputs: dict | None = None,
+        N_configs=1, 
+        max_pop = 20,
+        save_ensemble = False, 
+        cluster = None,
+        **kwargs
+    ):
         """
         This module initialize the relaxer. It may perform
         constant volume or pressure relaxation using fully anharmonic potentials.
@@ -63,6 +73,16 @@ class SSCHA(object):
             ase_calculator : ase.calculators...
                 An initialized ASE calculator. This will be used to compute energies and forces
                 for the relaxation of the SSCHA.
+            aiida_input: dict
+                Dictionary containing the input data for the 
+                :class:`~sscha.aiida_ensemble.AiiDAEnsemble.compute_ensmble`
+                method; namely:
+                    * pw_code: str,
+                    * protocol: str = 'moderate',
+                    * options: dict = None,
+                    * overrides: dict = None,
+                    * group_label: str = None,
+                    * kwargs
             N_configs : int
                 The number of configuration to be used for each population
             max_pop: int, optional
@@ -84,11 +104,13 @@ class SSCHA(object):
         self.N_configs = N_configs
         self.max_pop = max_pop
         self.cluster = cluster
+        self.aiida_inputs = aiida_inputs
         self.start_pop = 1
 
         # If the ensemble must be saved at each iteration.
         #
         self.save_ensemble = save_ensemble
+        self.save_dyn = True # Save the dynamical matrix at each iteration
         self.data_dir = "data"
 
 
@@ -350,8 +372,11 @@ Error, the specified location to save the ensemble:
                 self.minim.ensemble.generate(self.N_configs, sobol = sobol, sobol_scramble = sobol_scramble, sobol_scatter = sobol_scatter)
 
                 # Compute energies and forces
-                self.minim.ensemble.compute_ensemble(self.calc, get_stress,
-                                                 cluster = self.cluster)
+                if isinstance(self.minim.ensemble, AiiDAEnsemble):
+                    self.minim.ensemble.compute_ensemble(**self.aiida_inputs)
+                else:
+                    self.minim.ensemble.compute_ensemble(
+                        self.calc, get_stress, cluster = self.cluster)
                 #self.minim.ensemble.get_energy_forces(self.calc, get_stress)
 
                 if ensemble_loc is not None and self.save_ensemble:
@@ -379,7 +404,7 @@ Error, the specified location to save the ensemble:
                 self.minim.dyn.dynmats[iq] = fcq[iq, :, :]
 
             # Save the dynamical matrix
-            if self.save_ensemble:
+            if self.save_ensemble or self.save_dyn:
                 self.minim.dyn.save_qe("dyn_pop%d_" % pop)
 
             # Check if it is converged
@@ -551,6 +576,7 @@ Error, the specified location to save the ensemble:
             pop = start_pop
         else:
             pop = self.start_pop
+            start_pop = self.start_pop
 
         running = True
         while running:
@@ -558,13 +584,13 @@ Error, the specified location to save the ensemble:
             if kind_minimizer == "RPSD":
                 # Compute the static bulk modulus
                 sbm = GetStaticBulkModulus(self.minim.dyn.structure, self.calc)
-                print ("BM:")
-                print (sbm)
                 BFGS = sscha.Optimizer.SD_PREC_UC(self.minim.dyn.structure.unit_cell, sbm)
 
             # Generate the ensemble
             self.minim.ensemble.dyn_0 = self.minim.dyn.Copy()
             if pop != start_pop or not restart_from_ens:
+                #print("POP:", pop, "START_POP:", start_pop)
+                #print("RESTART_FROM_ENS:", restart_from_ens)
                 self.minim.ensemble.generate(self.N_configs, sobol=sobol, sobol_scramble = sobol_scramble, sobol_scatter = sobol_scatter)
 
                 # Save also the generation
@@ -572,20 +598,22 @@ Error, the specified location to save the ensemble:
                 #    self.minim.ensemble.save_bin(ensemble_loc, pop)
 
                 # Compute energies and forces
-                self.minim.ensemble.compute_ensemble(self.calc, True, stress_numerical,
-                                                 cluster = self.cluster)
+                if isinstance(self.minim.ensemble, AiiDAEnsemble):
+                    self.minim.ensemble.compute_ensemble(**self.aiida_inputs)
+                else:
+                    self.minim.ensemble.compute_ensemble(
+                        self.calc, True, stress_numerical, cluster = self.cluster)
                 #self.minim.ensemble.get_energy_forces(self.calc, True, stress_numerical = stress_numerical)
 
-                print("RELAX force length:", len(self.minim.ensemble.force_computed))
                 
                 if ensemble_loc is not None and self.save_ensemble:
                     self.minim.ensemble.save_bin(ensemble_loc, pop)
-                print("RELAX force length:", len(self.minim.ensemble.force_computed))
+
+
 
             self.minim.population = pop
             self.minim.init(delete_previous_data = False)
 
-            print("RELAX force length:", len(self.minim.ensemble.force_computed))
             self.minim.run(custom_function_pre = self.__cfpre__,
                            custom_function_post = self.__cfpost__,
                            custom_function_gradient = self.__cfg__)
@@ -599,9 +627,22 @@ Error, the specified location to save the ensemble:
             stress_tensor *= sscha.SchaMinimizer.__RyBohr3_to_evA3__
             stress_err *=  sscha.SchaMinimizer.__RyBohr3_to_evA3__
 
+            # Check if the stress tensor is actually loaded in the ensemble
+            if np.max(np.abs(self.minim.ensemble.stresses)) < 1e-10:
+                # Probably there is an error in the submission of the stress tensor calculation
+                raise ValueError("Error, the stress tensor is not loaded in the ensemble. Check the stress tensor calculation.")
+
             # Get the pressure
             Press = np.trace(stress_tensor) / 3
 
+            # Get the error correctly accounting for symmetries
+            stress_diagonal_err = np.diag(stress_err)
+            press_err = 0
+            for i in range(3):
+                if not stress_diagonal_err[i] in stress_diagonal_err[:i]:
+                    press_err += stress_diagonal_err[i]**2
+            press_err = np.sqrt(press_err)
+
             # Get the volume
             Vol = self.minim.dyn.structure.get_volume()
 
@@ -629,6 +670,11 @@ Error, the specified location to save the ensemble:
  ENTHALPIC CONTRIBUTION
  ======================
 
+ Current pressure P = {:.4f} +- {:.4f} GPa
+ Target pressure  P = {:.4f} GPa
+
+ For enthalpy we use the target pressure.
+
  P = {:.4f} GPa   V = {:.4f} A^3
 
  P V = {:.8e} eV
@@ -637,7 +683,7 @@ Error, the specified location to save the ensemble:
  Gibbs Free energy = {:.10e} eV {}
  Zero energy = {:.10e} eV
 
- """.format(target_press , Vol,target_press_evA3 * Vol, helmoltz, mark_helmoltz, gibbs, mark_gibbs, self.minim.eq_energy)
+ """.format(Press / CC.Units.GPA_TO_EV_PER_A3, press_err/ CC.Units.GPA_TO_EV_PER_A3, target_press, target_press, Vol, target_press_evA3 * Vol, helmoltz, mark_helmoltz, gibbs, mark_gibbs, self.minim.eq_energy)
             print(message)
             # print " ====================== "
             # print " ENTHALPIC CONTRIBUTION "
@@ -700,7 +746,8 @@ Error, the specified location to save the ensemble:
                 self.minim.dyn.dynmats[iq] = fcq[iq, :, :]
 
             # Save the dynamical matrix
-            self.minim.dyn.save_qe("dyn_pop%d_" % pop)
+            if self.save_ensemble or self.save_dyn:
+                self.minim.dyn.save_qe("dyn_pop%d_" % pop)
 
             # Check if the constant volume calculation is converged
             running1 = not self.minim.is_converged()
